﻿using System;
using System.Windows.Forms;
using SiMoPO;
using SiMoPO.HC;

namespace SimpleDemoCSharp
{
    public partial class Form1 : System.Windows.Forms.Form
    {

        IHillClimbingAlgorithms HillClimbingAlgorithm;

        const double MAXDOUBLE = 1.79769313486231E+308;
        int tfcall_counter;
        double Best = MAXDOUBLE;
        double ReturnValue;
        string RangStrTF;

        int startTicks;

        public Form1() : base()
        {
            Load += Form1_Load;

            //This call is required by the Windows Form Designer.
            InitializeComponent();

            //Add any initialization after the InitializeComponent() call
        }

        private void Form1_Load(System.Object sender, System.EventArgs e)
        {
            ButtonBreak.Enabled = false;

            TextX1.Text = (0.0).ToString();
            TextX2.Text = (0.0).ToString();
            TextX3.Text = (0.0).ToString();

            HillClimbingAlgorithm = HillClimbingAlgorithmsFactory.CreateObject(3);

            //Setup
            HillClimbingAlgorithm.Maximization = 0;            //minimize target function
            HillClimbingAlgorithm.Dimension = 3;               //number of independent parameters

            //set ranges and step width
            for (int i = 0; i <= HillClimbingAlgorithm.Dimension - 1; i++)
            {
                HillClimbingAlgorithm.OptiParaMin[i] = -100.0;
                HillClimbingAlgorithm.OptiParaMax[i] = 100.0;
                HillClimbingAlgorithm.OptiParaStep[i] = 0.01;
                //HillClimbingAlgorithm.OptiParaMaxIterate(i) = 0                 '0 is here default, you can use 1 when having integer options as parameters instead of linear adaptive step mode
                //HillClimbingAlgorithm.OptiParaStepAdaptiveMode(i) = 1           'all parameters are double values, exponential is default, otherwise choose here linear
                HillClimbingAlgorithm.OptiParaStepAdaptiveAutoScaling[i] = 1;
                HillClimbingAlgorithm.RandomizeStartParam[i] = 1;
            }
            HillClimbingAlgorithm.TargetFunction = new TargetFunctionCall(TargetFunctionOptimizationAlgorithm);

        }

        private void CalculateHillClimbing_Click(System.Object sender, System.EventArgs e)
        {
            StartOptimizationRun(HillClimbingAlgorithm);
        }

        private int StartOptimizationRun(IParameterOptimization OptimizationAlgorithm)
        {

            CalculateHillClimbing.Enabled = false;
            ButtonReset.Enabled = false;
            ButtonBreak.Enabled = true;
            TextX1.Enabled = false;
            TextX2.Enabled = false;
            TextX3.Enabled = false;

            tfcall_counter = 0;

            Best = MAXDOUBLE;

            OptimizationAlgorithm.BreakOptimization = 0;

            //set starting X parameters
            OptimizationAlgorithm.OptiPara[0] = Convert.ToDouble(TextX1.Text);
            OptimizationAlgorithm.OptiPara[1] = Convert.ToDouble(TextX2.Text);
            OptimizationAlgorithm.OptiPara[2] = Convert.ToDouble(TextX3.Text);

            startTicks = Environment.TickCount;

            //start optimization run
            OptimizationAlgorithm.Start();

            //set optimum X values
            TextX1.Text = OptimizationAlgorithm.OptiParaOptimum[0].ToString();
            TextX2.Text = OptimizationAlgorithm.OptiParaOptimum[1].ToString();
            TextX3.Text = OptimizationAlgorithm.OptiParaOptimum[2].ToString();


            RangStrTF = "Demo Optimization" + "      Best: " + Best.ToString() + "      Evaluations: " + tfcall_counter.ToString() + "      Time: " + TimeGoneBy(startTicks).ToString() + " s";
            ToolStripStatusLabel1.Text = RangStrTF;

            TextX1.Enabled = true;
            TextX2.Enabled = true;
            TextX3.Enabled = true;
            ButtonBreak.Enabled = false;
            ButtonReset.Enabled = true;
            CalculateHillClimbing.Enabled = true;

            return 0;
        }

        public double TargetFunctionOptimizationAlgorithm(IParameterOptimization OptimizationAlgorithm)
        {
            ReturnValue = Math.Pow((OptimizationAlgorithm.OptiPara[0] - 10), 2) + Math.Pow((OptimizationAlgorithm.OptiPara[1] - 20), 2) + Math.Pow((OptimizationAlgorithm.OptiPara[2] - 30), 2);

            if (ReturnValue < Best)
            {
                Best = ReturnValue;
            }

            if (tfcall_counter % 25 == 0)
            {
                Application.DoEvents();
                RangStrTF = "Demo Optimization" + "      Best: " + Best.ToString() + "      Actual: " + ReturnValue.ToString() + "      Evaluations: " + tfcall_counter.ToString() + "      Time: " + TimeGoneBy(startTicks).ToString() + " s";
                ToolStripStatusLabel1.Text = RangStrTF;
            }

            try
            {
                (OptimizationAlgorithm as IHillClimbingAlgorithms).InfeasibleSolution = 0;
            }
            catch (Exception ex)
            {
                //MessageBox.Show(ex.Message);
            }

            tfcall_counter += 1;

            return ReturnValue;
        }

        private void ButtonBreak_Click(System.Object sender, System.EventArgs e)
        {
            HillClimbingAlgorithm.BreakOptimization = 1;
        }

        private double TimeGoneBy(int startTicks)
        {
            // This function throws an overflow exception if the
            // tick count difference is greater than 2,147,483,647,  
            // about 24 days for My.Computer.Clock.TickCount.

            // Use UInteger to simplify the code for roll over.
            uint uStart = Convert.ToUInt32(Convert.ToInt64(startTicks) - int.MinValue);
            uint uCurrent = Convert.ToUInt32(Convert.ToInt64(Environment.TickCount) - int.MinValue);

            // Calculate the tick count difference.
            uint tickCountDifference = 0;
            if (uStart <= uCurrent)
            {
                tickCountDifference = uCurrent - uStart;
            }
            else
            {
                // Tick count rolled over.
                tickCountDifference = uint.MaxValue - (uStart - uCurrent);
            }

            // Convert milliseconds to seconds
            return Convert.ToInt32(tickCountDifference) / 1000.0;
        }

        private void Reset_Click(System.Object sender, System.EventArgs e)
        {
            TextX1.Text = (0.0).ToString();
            TextX2.Text = (0.0).ToString();
            TextX3.Text = (0.0).ToString();
        }
    }
}
