Imports SiMoPO
Imports SiMoPO.HC

Public Class Form1

    Dim HillClimbingAlgorithm As IHillClimbingAlgorithms

    Const MAXDOUBLE As Double = 1.79769313486231E+308
    Dim tfcall_counter As Integer
    Dim Best As Double = MAXDOUBLE
    Dim ReturnValue As Double
    Dim RangStrTF As String
    Dim startTicks As Integer

    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        ButtonBreak.Enabled = False

        TextX1.Text = (0.0).ToString
        TextX2.Text = (0.0).ToString
        TextX3.Text = (0.0).ToString

        HillClimbingAlgorithm = HillClimbingAlgorithmsFactory.CreateObject(3)

        With HillClimbingAlgorithm

            'Setup
            .Maximization = 0                   'minimize target function
            .Dimension = 3                      'number of independent parameters

            'set ranges and step width
            For i As Integer = 0 To .Dimension - 1
                .OptiParaMin(i) = -100.0
                .OptiParaMax(i) = 100.0
                .OptiParaStep(i) = 0.01
                '.OptiParaMaxIterate(i) = 0                 '0 is here default, you can use 1 when having integer options as parameters instead of linear adaptive step mode
                '.OptiParaStepAdaptiveMode(i) = 1           'all parameters are double values, exponential is default, otherwise choose here linear
                .OptiParaStepAdaptiveAutoScaling(i) = 1
                .RandomizeStartParam(i) = 1
            Next
            .TargetFunction = AddressOf TargetFunctionOptimizationAlgorithm

        End With

    End Sub

    Private Sub CalculateHillClimbing_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CalculateHillClimbing.Click
        StartOptimizationRun(HillClimbingAlgorithm)
    End Sub


    Private Function StartOptimizationRun(ByVal OptimizationAlgorithm As IParameterOptimization) As Integer

        CalculateHillClimbing.Enabled = False
        ButtonReset.Enabled = False
        ButtonBreak.Enabled = True
        TextX1.Enabled = False
        TextX2.Enabled = False
        TextX3.Enabled = False

        tfcall_counter = 0

        Best = MAXDOUBLE

        With OptimizationAlgorithm

            .BreakOptimization = 0

            'set starting X parameters
            .OptiPara(0) = CDbl(TextX1.Text)
            .OptiPara(1) = CDbl(TextX2.Text)
            .OptiPara(2) = CDbl(TextX3.Text)

            startTicks = My.Computer.Clock.TickCount

            'start optimization run
            .Start()

            'set optimum X values
            TextX1.Text = .OptiParaOptimum(0).ToString
            TextX2.Text = .OptiParaOptimum(1).ToString
            TextX3.Text = .OptiParaOptimum(2).ToString

        End With

        RangStrTF = "Demo Optimization" + "      Best: " + Best.ToString + "      Evaluations: " + tfcall_counter.ToString + "      Time: " + TimeGoneBy(startTicks).ToString + " s"
        ToolStripStatusLabel1.Text = RangStrTF

        TextX1.Enabled = True
        TextX2.Enabled = True
        TextX3.Enabled = True
        ButtonBreak.Enabled = False
        ButtonReset.Enabled = True
        CalculateHillClimbing.Enabled = True

        Return 0

    End Function

    Public Function TargetFunctionOptimizationAlgorithm(ByVal OptimizationAlgorithm As IParameterOptimization) As Double

        With OptimizationAlgorithm
            ReturnValue = (.OptiPara(0) - 10) ^ 2 + (.OptiPara(1) - 20) ^ 2 + (.OptiPara(2) - 30) ^ 2
        End With

        If ReturnValue < Best Then
            Best = ReturnValue
        End If

        If tfcall_counter Mod 25 = 0 Then
            Application.DoEvents()
            RangStrTF = "Demo Optimization" + "      Best: " + Best.ToString + "      Actual: " + ReturnValue.ToString + "      Evaluations: " + tfcall_counter.ToString + "      Time: " + TimeGoneBy(startTicks).ToString + " s"
            ToolStripStatusLabel1.Text = RangStrTF
        End If

        Try
            TryCast(OptimizationAlgorithm, IHillClimbingAlgorithms).InfeasibleSolution = 0
        Catch ex As Exception
        End Try

        tfcall_counter += 1

        Return ReturnValue

    End Function

    Private Sub ButtonBreak_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonBreak.Click
        HillClimbingAlgorithm.BreakOptimization = 1
    End Sub

    Private Function TimeGoneBy(ByVal startTicks As Integer) As Double
        ' This function throws an overflow exception if the
        ' tick count difference is greater than 2,147,483,647,  
        ' about 24 days for My.Computer.Clock.TickCount.

        ' Use UInteger to simplify the code for roll over.
        Dim uStart As UInteger = _
            CUInt(CLng(startTicks) - Integer.MinValue)
        Dim uCurrent As UInteger = _
            CUInt(CLng(My.Computer.Clock.TickCount) - Integer.MinValue)

        ' Calculate the tick count difference.
        Dim tickCountDifference As UInteger
        If uStart <= uCurrent Then
            tickCountDifference = uCurrent - uStart
        Else
            ' Tick count rolled over.
            tickCountDifference = UInteger.MaxValue - (uStart - uCurrent)
        End If

        ' Convert milliseconds to seconds
        Return CInt(tickCountDifference) / 1000
    End Function

    Private Sub Reset_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonReset.Click
        TextX1.Text = (0.0).ToString
        TextX2.Text = (0.0).ToString
        TextX3.Text = (0.0).ToString
    End Sub
End Class
